classdef FysAnalyzerClass < handle
    
    properties (SetObservable = true)
        datafilename = '' %Path + filename of the file with raw perfusion data
        bpfilename = ''; % CSV file containing blood pressure data in 2nd column
        %numberOfBPSamples = [];
        treshold = 100; % perfusion treshold for generating mask for filtering pixels for calculation
        pad = 10; %Number to exclude from fit in each end
        maxBP = 40; %Max blodpressure to try and find knot
        roofBP = 90; %Max bloodpressure to fit line segment to
        BPresolution =1; %Stepsize for range of BP to test for optimal knot.
        blur = 1.5; %The gaussian blur applied to the picture
        %cancel = 0; %Switched to 0 if cancel button pressed in waitbars
        selX = 1; %The currently selected X pixel
        selY = 1; %The currently selected Y pixel
        csv_delimiter = ';'; %Delimiter used for parsing CSV file
        csv_columnname = 'MAP'; %Name of the column which contains the bloodpressure data
    end
    
    properties
        handles = struct; %Struct with handles for images, plot elements etc.
        pictures = struct(); %Struct with the generated pictures
        abp = []; %The bloodpressure data fitting with the common length
        commonlength = []; %Minimum length of image data and blood pressure
        dataCube = []; %Trimmed Image Data: Only images fitting with the length of BP data and blurred
        csvtb = table(); % Table containing data from CSV file
        MAP = []; %All the BP values
        imagedata = []; %All images, not trimmed.
        PIM = PIMSoftBinary(); %The object used for reading the raw perfusion data
        mask = []; %Pixels that should be fitted to, calculated from first 10 pics above treshold
    end
    
    properties ( Access = 'private' )
        fid = -1;
    end
    
    events
        changedCoord
    end
    
    methods       
        function PerfusionImage = calcPerfusion(obj, frameNumber)
            VarianceImage = obj.PIM.getVarianceFrame(frameNumber);
            IntensityImage = obj.PIM.getDCFrame(frameNumber);
            CoherenceFactor = obj.PIM.coherenceFactor;
            SignalGain =obj.PIM.signalGain;
            WindowSize = 1;
            PerfusionImage = calcPerfusion(VarianceImage,IntensityImage,CoherenceFactor,SignalGain,WindowSize);
            
        end
        
        function makeDatacube(obj);
            w = waitbar(0, 'Calculating Perfusion Data');
            obj.imagedata= zeros(obj.PIM.imageHeight,obj.PIM.imageWidth, obj.PIM.numberOfImages);
            for i = 1:obj.PIM.numberOfImages
                obj.imagedata(:,:,i) = imgaussfilt(obj.calcPerfusion(i),obj.blur);
                waitbar(i/obj.PIM.numberOfImages);
            end;
            close(w);
        end
		
		function readBP(obj, filename);
			%mat = dlmread(filename, ';', 1,1);
            %obj.MAP = mat(:,1);
            obj.csvtb = readtable(filename,'Delimiter',obj.csv_delimiter);
            obj.MAP = obj.csvtb.(obj.csv_columnname);
            obj.bpfilename = filename;
            obj.dataread();
        end
        
        function readDatafile(obj, filename);
            obj.PIM.OpenFile(filename);
            obj.datafilename = filename;
            obj.dataread();
        end
        
        function dataread(obj); %Executed when read in data is updated. Triggers trimming and calculation of dataCube.
            if obj.PIM.numberOfImages > 0 & size(obj.MAP,2) > 0
                obj.makeDatacube();
                obj.trimDatacube();
                obj.makemask(obj.treshold);
            end
        end
        
        function commonlength = get.commonlength(obj);
            commonlength = min(size(obj.MAP,1),size(obj.imagedata,3));
        end
        
        function abp = get.abp(obj);
            abp = obj.MAP(1:obj.commonlength);
        end
        
        function set.selY(obj,selY);
            obj.selY = selY;
            notify(obj, 'changedCoord');
        end
        
        function set.selX(obj,selX);
            obj.selX = selX;
            notify(obj,'changedCoord');
        end
        
        %function result = get.dataCube(obj);
        %    result = obj.imagedata(:,:,1:obj.commonlength);
        %end
        function trimDatacube(obj)
            %TODO: We should apply the Gaussian Blue here, and leave the raw data
            %in the imagedata.
            obj.dataCube = obj.imagedata(:,:,1:obj.commonlength);
        end
        
        function result = extractPixel(obj,X,Y) %Extract the data for a single Pixel.
            %result = squeeze(obj.dataCube(X,Y,:));
            result = reshape(obj.dataCube(X,Y,:),[obj.commonlength,1]);
            result = result/mean(result(1:10))*100;
        end
        
        function [coeffs, coeffs2,knot,cost] = findknee(obj,Xpx,Ypx)
            pad = obj.pad; 
            %Fit a line
            x = obj.abp;
            y = obj.extractPixel(Xpx,Ypx);
            %sort x and y (still necessary??)
            %[x, I] = sort(x);
            %y= y(I);
            %Brute Force find minimum X
            costf = [];
            range = round(x(end-pad)):obj.BPresolution:obj.maxBP;
            %display(range);
            for i = range%i = pad:obj.commonlength-pad
                %display(x(i));
                XI = [min(x),i,max(x)];
                YI = lsq_lut_piecewise( x, y, XI );
                %Costfunction 
               costf = [costf ; sum((y-interp1(XI,YI,x)).^2)];
            end
            %Identify minimum costf
            [cost,I] = min(costf);
            %i = I + pad; %Re add the padding
            i = range(I);
            %Identify the pieces
            XI = [min(x),i,obj.roofBP];
            YI = lsq_lut_piecewise( x, y, XI );

            %Calculate the slope and intercepts
            slope = (YI(1)-YI(2))/(XI(1)-XI(2));
            slope2 = (YI(2)-YI(3))/(XI(2)-XI(3));
            intercept = YI(1)-slope*XI(1);
            intercept2 = YI(2)-slope2*XI(2);
            %Return the results
            coeffs = [slope,intercept];
            coeffs2 = [slope2,intercept2];
            knot = i;
        end
        
        function makemask(obj, threshold) %Make a mask for the pixels not to calculate
            obj.mask = mean(obj.imagedata(:,:,1:10),3) > threshold;
        end
        
        function fitLineSegments(obj);
            %Prepare Matrices
            obj.pictures.slope1 = zeros(obj.PIM.imageHeight,obj.PIM.imageWidth);
            obj.pictures.slope2 = zeros(obj.PIM.imageHeight,obj.PIM.imageWidth);
            obj.pictures.intercept1 = zeros(obj.PIM.imageHeight,obj.PIM.imageWidth);
            obj.pictures.intercept2 = zeros(obj.PIM.imageHeight,obj.PIM.imageWidth);
            obj.pictures.knotpic = zeros(obj.PIM.imageHeight,obj.PIM.imageWidth);
            obj.pictures.cost = zeros(obj.PIM.imageHeight,obj.PIM.imageWidth);
            w2 = waitbar(0,'Fitting Line Segments');
            for h = 1:obj.PIM.imageHeight
                for w = 1:obj.PIM.imageWidth
                    if obj.mask(h,w) == 1;
                        [coeffs,coeffs2,knot,cost] = obj.findknee(h,w);
                        obj.pictures.knotpic(h,w) = knot;
                        obj.pictures.slope1(h,w) = coeffs(1);
                        obj.pictures.intercept1(h,w) = coeffs(2);
                        obj.pictures.slope2(h,w) = coeffs2(1);
                        obj.pictures.intercept2(h,w) = coeffs2(2);
                        obj.pictures.cost(h,w) = cost;
                    end;
                end;
            waitbar(h/obj.PIM.imageHeight,w2);
            end;
            close(w2);
        end
        
        function save_calculation(obj, path, filename)
            savestruct = struct();
            savestruct.pad = obj.pad;
            savestruct.datafilename = obj.datafilename;
            savestruct.bpfilename = obj.bpfilename;
            savestruct.mask = obj.mask;
            savestruct.pad = obj.pad;
            savestruct.maxBP = obj.maxBP; %Max blodpressure to try an find knot
            savestruct.BPresolution = obj.BPresolution; %Stepsize for range of BP to test for optimal knot.
            savestruct.treshold = obj.treshold; % perfusion treshold for generating mask for filtering pixels for calculation
            savestruct.pictures = obj.pictures;
            save(strcat(path,filename),'savestruct');
        end
        
        function load_calculation(obj, path, filename)
            load(strcat(path,filename),'savestruct');
            obj.pad = savestruct.pad;
            obj.datafilename = savestruct.datafilename ;
            obj.bpfilename = savestruct.bpfilename;
            obj.mask = savestruct.mask;
            obj.pad = savestruct.pad;
            obj.maxBP = savestruct.maxBP; %Max blodpressure to try an find knot
            obj.BPresolution = savestruct.BPresolution; %Stepsize for range of BP to test for optimal knot.
            obj.treshold = savestruct.treshold; % perfusion treshold for generating mask for filtering pixels for calculation
            obj.pictures = savestruct.pictures;
            %Load previous datafiles if available
            %obj.bpfilename = '';
            %obj.datafilename = '';
            %obj.readBP(obj.bpfilename);
            %obj.readDatafile(obj.datafilename);
        end
        
        function save_datacube(obj)
            [filename, path] = uiputfile('*.mat','Please choose a filename for .mat file');
            if ~isequal(filename,0)
               obj.save_datacube_filename(path, filename);
            end
        end
        
        function save_datacube_filename(obj, path, filename)
            if ~isequal(filename,0)
                dataCube = obj.dataCube;
                MAP = obj.MAP;
                save(strcat(path,filename),'dataCube','MAP');
            end
        end
        
        function load_datacube(obj)
            [filename, path] = uigetfile('*.mat','Please choose Datacube .mat file');
            if ~isequal(filename,0)
                obj.load_datacube_filename(path, filename);
            end
        end
        
        function load_datacube_filename(obj, path, filename)
            if ~isequal(filename,0)
                load(strcat(path,filename),'dataCube','MAP');
                obj.dataCube = dataCube;
                obj.MAP = MAP;
            end
        end
        
        
    end; %Methods
end

