function [ PerfusionImage ] = calcPerfusion( VarianceImage, IntensityImage, CoherenceFactor, SignalGain, WindowSize)
% [ PerfusionImage ] = calcPerfusion( VarianceImage, IntensityImage,...
% CoherenceFactor, SignalGain, WindowSize)
% This function calculates the perfusion from a variance and an intensity
% image. The coherence factor is needed to scale the contrast, if it is not
% given, a default value of 2.5 will be used. The signal gain is needed to
% scale the perfusion, if it is not given, a default value of 95 will be
% used. If WindowSize is given, the perfusion will be scaled by averaging 
% over blocks of n*n pixels, where n=windowSize. This is done by first 
% downscaling the variance and intensity images and then calculating the 
% perfusion, because scaling down the perfusion image directly can give 
% incorrect values.

% Version 1.0   Perimed 2010

if nargin<2
    errordlg('Error: perfusion calculation needs variance and intensity image.')
    PerfusionImage=[];
    return
elseif nargin==2
    warndlg('Warning: no coherence factor and signal gain specified. Assuming Beta=2.5 and k=95.')
    CoherenceFactor=2.5;
    SignalGain=95;
elseif nargin==3
    warndlg('Warning: no signal gain specified. Assuming 95.')
    SignalGain=95;
end

if nargin==4||WindowSize==1 % Calculation of single values
    VarSign=sign(VarianceImage);
    ContrastImage = sqrt(abs(VarianceImage))./IntensityImage*CoherenceFactor;
    ContrastImage = VarSign.*ContrastImage;
    ContrastImage(ContrastImage<=0) = nan;
    PerfusionImage = SignalGain * (1./ContrastImage - 1);
    PerfusionImage(isnan(PerfusionImage)) = 3000;
    PerfusionImage(PerfusionImage>3000) = 3000;
else                        % Downscale image
    AveragedVarianceImage = blockAverageDownscale(VarianceImage, WindowSize);
    VarSign=sign(AveragedVarianceImage);
    AveragedIntensityImage = blockAverageDownscale(IntensityImage, WindowSize);
    AveragedContrastImage = sqrt(abs(AveragedVarianceImage))./AveragedIntensityImage*CoherenceFactor;
    AveragedContrastImage = VarSign.*AveragedContrastImage;
    AveragedContrastImage(AveragedContrastImage<=0) = nan;
    PerfusionImage = SignalGain * (1./AveragedContrastImage - 1);
    PerfusionImage(isnan(PerfusionImage)) = 3000;
    PerfusionImage(PerfusionImage>3000) = 3000;
end

end

